"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var measure_1 = require("../measure");
var testUtils_1 = require("./testUtils");
describe("Measures", function () {
    var meter = measure_1.Measure.dimension("L");
    var second = measure_1.Measure.dimension("T");
    var mps = meter.per(second);
    var mps2 = mps.per(second);
    describe("dimension", function () {
        it("should create dimensions with value 1", function () {
            expect(measure_1.Measure.dimension("foo", "f")).toEqual({ value: 1, unit: { foo: ["f", 1] }, symbol: "f" });
        });
    });
    describe("construction", function () {
        it("should construct from a number of and a unit", function () {
            var measure = measure_1.Measure.of(10, mps2);
            expect(measure.value).toBe(10);
            expect(measure.unit).toEqual(mps2.unit);
        });
        it("should construct from a number and another measure", function () {
            var kilometer = measure_1.Measure.of(1000, meter);
            var measure = measure_1.Measure.of(5.2, kilometer);
            expect(measure.value).toBe(5200);
            expect(measure.unit).toEqual(meter.unit);
        });
        it("should construct dimensionless values", function () {
            var dimensionless = measure_1.Measure.dimensionless(3);
            expect(dimensionless.value).toBe(3);
            expect(dimensionless.unit).toEqual({});
        });
    });
    describe("arithmetic", function () {
        it("should negate", function () {
            var value = measure_1.Measure.of(10, mps);
            expect(value.negate()).toEqual(measure_1.Measure.of(-10, mps));
        });
        it("should add", function () {
            var left = measure_1.Measure.of(10, mps);
            var right = measure_1.Measure.of(5, mps);
            expect(left.plus(right)).toEqual(measure_1.Measure.of(15, mps));
        });
        it("should subtract", function () {
            var left = measure_1.Measure.of(10, second);
            var right = measure_1.Measure.of(5, second);
            expect(left.minus(right)).toEqual(measure_1.Measure.of(5, second));
        });
        it("should multiply", function () {
            var left = measure_1.Measure.of(10, mps);
            var right = measure_1.Measure.of(5, second);
            expect(left.times(right)).toEqual(measure_1.Measure.of(50, meter));
        });
        it("should divide", function () {
            var left = measure_1.Measure.of(10, mps);
            var right = measure_1.Measure.of(5, second);
            expect(left.over(right)).toEqual(measure_1.Measure.of(2, mps2));
            expect(left.per(right)).toEqual(measure_1.Measure.of(2, mps2));
        });
        it("should scale", function () {
            var value = measure_1.Measure.of(10, mps);
            var three = measure_1.Measure.dimensionless(3);
            expect(value.scale(2)).toEqual(measure_1.Measure.of(20, mps));
            expect(value.scale(three)).toEqual(measure_1.Measure.of(30, mps));
        });
        it("should exponentiate", function () {
            var value = measure_1.Measure.of(10, meter);
            expect(value.inverse()).toEqual(measure_1.Measure.of(0.1, meter.inverse()));
            expect(value.reciprocal()).toEqual(measure_1.Measure.of(0.1, meter.inverse()));
            expect(value.pow(0)).toEqual(measure_1.Measure.dimensionless(1));
            expect(value.pow(1)).toEqual(measure_1.Measure.of(10, meter));
            expect(value.squared()).toEqual(measure_1.Measure.of(100, meter.squared()));
            expect(value.cubed()).toEqual(measure_1.Measure.of(1000, meter.cubed()));
        });
    });
    describe("comparison", function () {
        var zero = measure_1.Measure.of(0, meter);
        var five = measure_1.Measure.of(5, meter);
        var ten = measure_1.Measure.of(10, meter);
        it("should compare less than", function () {
            expect(five.isLessThan(zero)).toBe(false);
            expect(five.isLessThan(five)).toBe(false);
            expect(five.isLessThan(ten)).toBe(true);
        });
        it("should compare less than or equal to", function () {
            expect(five.isLessThanOrEqualTo(zero)).toBe(false);
            expect(five.isLessThanOrEqualTo(five)).toBe(true);
            expect(five.isLessThanOrEqualTo(ten)).toBe(true);
        });
        it("should compare equal to", function () {
            expect(five.isEqualTo(zero)).toBe(false);
            expect(five.isEqualTo(five)).toBe(true);
            expect(five.isEqualTo(ten)).toBe(false);
        });
        it("should compare not equal to", function () {
            expect(five.isNotEqualTo(zero)).toBe(true);
            expect(five.isNotEqualTo(five)).toBe(false);
            expect(five.isNotEqualTo(ten)).toBe(true);
        });
        it("should compare greater than or equal to", function () {
            expect(five.isGreaterThanOrEqualTo(zero)).toBe(true);
            expect(five.isGreaterThanOrEqualTo(five)).toBe(true);
            expect(five.isGreaterThanOrEqualTo(ten)).toBe(false);
        });
        it("should compare greater than", function () {
            expect(five.isGreaterThan(zero)).toBe(true);
            expect(five.isGreaterThan(five)).toBe(false);
            expect(five.isGreaterThan(ten)).toBe(false);
        });
    });
    describe("symbols", function () {
        it("should assign a symbol via .of()", function () {
            expect(measure_1.Measure.of(1000, meter, "km").symbol).toBe("km");
        });
        it("should copy assign a symbol via .withSymbol()", function () {
            var original = measure_1.Measure.of(1000, meter);
            var result = original.withSymbol("km");
            expect(result).not.toBe(original);
            expect(original.symbol).toBeUndefined();
            expect(result.symbol).toBe("km");
        });
        it("should not pass along symbols through operations", function () {
            var km = measure_1.Measure.of(1000, meter.squared()).withSymbol("km2");
            var dm = measure_1.Measure.of(10, meter.squared()).withSymbol("dm2");
            expect(km.negate().symbol).toBeUndefined();
            expect(km.squared().symbol).toBeUndefined();
            expect(km.inverse().symbol).toBeUndefined();
            expect(km.plus(dm).symbol).toBeUndefined();
            expect(km.minus(dm).symbol).toBeUndefined();
            expect(km.times(dm).symbol).toBeUndefined();
            expect(km.over(dm).symbol).toBeUndefined();
        });
    });
    describe("formatting", function () {
        it("should format dimensionless units", function () {
            expect(measure_1.Measure.dimensionless(10).toString()).toBe("10");
            expect(measure_1.Measure.unsafeConstruct(10, testUtils_1.addSymbols({ x: 0, y: undefined })).toString()).toBe("10");
        });
        it("should format base units", function () {
            expect(meter.toString()).toBe("1 L");
            expect(measure_1.Measure.of(5.3, meter).toString()).toBe("5.3 L");
        });
        it("should format complex units", function () {
            expect(measure_1.Measure.of(5, meter.squared()).toString()).toBe("5 L^2");
            expect(measure_1.Measure.of(5, second.inverse()).toString()).toBe("5 T^-1");
            expect(measure_1.Measure.of(5, meter.times(second)).toString()).toBe("5 L * T");
            expect(measure_1.Measure.of(5, meter.over(second)).toString()).toBe("5 L * T^-1");
            expect(measure_1.Measure.of(5, meter.cubed().over(second)).toString()).toBe("5 L^3 * T^-1");
            expect(measure_1.Measure.of(5, meter.cubed().over(second.squared())).toString()).toBe("5 L^3 * T^-2");
        });
        it("should have consistent formatting no matter how the unit is constructed", function () {
            var metersTimesSecond = "5 L * T";
            expect(measure_1.Measure.of(5, meter.times(second)).toString()).toBe(metersTimesSecond);
            expect(measure_1.Measure.of(5, second.times(meter)).toString()).toBe(metersTimesSecond);
            var metersPerSecond = "5 L * T^-1";
            expect(measure_1.Measure.of(5, meter.per(second)).toString()).toBe(metersPerSecond);
            expect(measure_1.Measure.of(5, second.inverse().times(meter)).toString()).toBe(metersPerSecond);
        });
        it("should not format using symbol even if present", function () {
            expect(measure_1.Measure.of(5, meter.squared())
                .withSymbol("m2")
                .toString()).toBe("5 L^2");
            expect(measure_1.Measure.dimensionless(0)
                .withSymbol("rad")
                .toString()).toBe("0");
        });
        it("should skip formatting explicitly 0 and undefined dimension", function () {
            expect(measure_1.Measure.unsafeConstruct(10, testUtils_1.addSymbols({ x: 0, y: undefined, z: 2 })).toString()).toBe("10 z^2");
        });
        it("should format measures as other measures with symbols", function () {
            var glorbs = measure_1.Measure.of(100, meter, "glb");
            expect(measure_1.Measure.of(1000, meter).in(glorbs)).toBe("10 glb");
        });
        it("should use normal formatting if the other measure has no symbol", function () {
            var glorbs = measure_1.Measure.of(100, meter);
            expect(measure_1.Measure.of(1000, meter).in(glorbs)).toBe("1000 L");
        });
        it("should use base unit symbols to format when available", function () {
            var m = measure_1.Measure.dimension("test-length", "meter");
            var s = measure_1.Measure.dimension("test-time", "second");
            expect(m.toString()).toBe("1 meter");
            expect(measure_1.Measure.of(1, m.per(s)).toString()).toBe("1 meter * second^-1");
            expect(measure_1.Measure.of(1, m.squared().per(s.squared())).toString()).toBe("1 meter^2 * second^-2");
        });
    });
});
//# sourceMappingURL=measureTests.js.map