"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var format_1 = require("./format");
var unitValueArithmetic_1 = require("./unitValueArithmetic");
/**
 * Creates a new measure factory for a given numeric type. The numeric type of the measure is inferred from the
 * parameter.
 * @param num the set of numeric operations needed to implement a measure for an arbitrary numeric type
 * @returns a factory for constructing measures of the given numeric type
 * @example
 * type MyMeasure<U extends Unit> = GenericMeasure<U, MyNumberType>;
 * const MyMeasure = createMeasureType({ ... });
 */
function createMeasureType(num) {
    var InternalMeasure = /** @class */ (function () {
        function InternalMeasure(value, unit, symbol) {
            this.value = value;
            this.unit = unit;
            this.symbol = symbol;
        }
        InternalMeasure.prototype.withSymbol = function (symbol) {
            return new InternalMeasure(this.value, this.unit, symbol);
        };
        // Arithmetic
        InternalMeasure.prototype.plus = function (other) {
            return new InternalMeasure(num.add(this.value, other.value), this.unit);
        };
        InternalMeasure.prototype.minus = function (other) {
            return new InternalMeasure(num.sub(this.value, other.value), this.unit);
        };
        InternalMeasure.prototype.negate = function () {
            return new InternalMeasure(num.neg(this.value), this.unit);
        };
        InternalMeasure.prototype.scale = function (value) {
            var numericValue = num.guard(value) ? value : value.value;
            return new InternalMeasure(num.mult(numericValue, this.value), this.unit);
        };
        InternalMeasure.prototype.times = function (other) {
            return new InternalMeasure(num.mult(this.value, other.value), unitValueArithmetic_1.multiplyUnits(this.unit, other.unit));
        };
        InternalMeasure.prototype.over = function (other) {
            return new InternalMeasure(num.div(this.value, other.value), unitValueArithmetic_1.divideUnits(this.unit, other.unit));
        };
        InternalMeasure.prototype.per = function (other) {
            return this.over(other);
        };
        InternalMeasure.prototype.pow = function (power) {
            return new InternalMeasure(num.pow(this.value, power), unitValueArithmetic_1.exponentiateUnit(this.unit, power));
        };
        InternalMeasure.prototype.inverse = function () {
            return new InternalMeasure(num.div(num.one(), this.value), unitValueArithmetic_1.exponentiateUnit(this.unit, -1));
        };
        InternalMeasure.prototype.reciprocal = function () {
            return this.inverse();
        };
        InternalMeasure.prototype.unsafeMap = function (fn) {
            return new InternalMeasure(fn(this.value), this.unit);
        };
        // Comparisons
        InternalMeasure.prototype.isLessThan = function (other) {
            return num.lt(this.value, other.value);
        };
        InternalMeasure.prototype.isLessThanOrEqualTo = function (other) {
            return num.lte(this.value, other.value);
        };
        InternalMeasure.prototype.isEqualTo = function (other) {
            return num.eq(this.value, other.value);
        };
        InternalMeasure.prototype.isNotEqualTo = function (other) {
            return num.neq(this.value, other.value);
        };
        InternalMeasure.prototype.isGreaterThanOrEqualTo = function (other) {
            return num.gte(this.value, other.value);
        };
        InternalMeasure.prototype.isGreaterThan = function (other) {
            return num.gt(this.value, other.value);
        };
        // Formatting
        InternalMeasure.prototype.toString = function () {
            return "" + num.format(this.value) + format_1.formatUnit(this.unit);
        };
        InternalMeasure.prototype.in = function (unit) {
            if (unit.symbol === undefined) {
                return this.toString();
            }
            var value = num.format(num.div(this.value, unit.value));
            return value + " " + unit.symbol;
        };
        return InternalMeasure;
    }());
    InternalMeasure.prototype.squared = function () {
        return this.pow(2);
    };
    InternalMeasure.prototype.cubed = function () {
        return this.pow(3);
    };
    return {
        class: InternalMeasure,
        dimension: function (dim, symbol) {
            return new InternalMeasure(num.one(), unitValueArithmetic_1.dimension(dim, symbol), symbol);
        },
        dimensionless: function (value) {
            return new InternalMeasure(value, {});
        },
        of: function (value, quantity, symbol) {
            return new InternalMeasure(num.mult(value, quantity.value), quantity.unit, symbol);
        },
        unsafeConstruct: function (value, unit, symbol) {
            return new InternalMeasure(value, unit, symbol);
        },
    };
}
exports.createMeasureType = createMeasureType;
//# sourceMappingURL=genericMeasure.js.map