"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("../unit/base");
var measure_1 = require("./measure");
var unitValueArithmetic_1 = require("./unitValueArithmetic");
/** `Math.abs` for `number` measures. The measure's unit is preserved under this operation. */
exports.abs = wrapUnaryFn(Math.abs);
/** `Math.ceil` for `number` measures. The measure's unit is preserved under this operation. */
exports.ceil = wrapUnaryFn(Math.ceil);
/** `Math.floor` for `number` measures. The measure's unit is preserved under this operation. */
exports.floor = wrapUnaryFn(Math.floor);
/** `Math.fround` for `number` measures. The measure's unit is preserved under this operation. */
exports.fround = wrapUnaryFn(Math.fround);
/** `Math.round` for `number` measures. The measure's unit is preserved under this operation. */
exports.round = wrapUnaryFn(Math.round);
/** `Math.trunc` for `number` measures. The measure's unit is preserved under this operation. */
exports.trunc = wrapUnaryFn(Math.trunc);
/** `Math.hypot` for `number` measures. At least 1 parameter must be provided and all must have the same unit. */
exports.hypot = warpNary(Math.hypot);
/** `Math.max` for `number` measures. At least 1 parameter must be provided and all must have the same unit. */
exports.max = warpNary(Math.max);
/** `Math.min` for `number` measures. At least 1 parameter must be provided and all must have the same unit. */
exports.min = warpNary(Math.min);
/**
 * `Math.acos` for `number` measures.
 * @param x a dimensionless value
 * @returns an angle
 */
exports.acos = wrapInverseTrigFn(Math.acos);
/**
 * `Math.asin` for `number` measures.
 * @param x a dimensionless value
 * @returns an angle
 */
exports.asin = wrapInverseTrigFn(Math.asin);
/**
 * `Math.atan` for `number` measures.
 * @param x a dimensionless value
 * @returns an angle
 */
exports.atan = wrapInverseTrigFn(Math.atan);
/**
 * `Math.cos` for `number` measures.
 * @param x an angle
 * @returns a dimensionless value
 */
exports.cos = wrapTrigFn(Math.cos);
/**
 * `Math.sin` for `number` measures.
 * @param x an angle
 * @returns a dimensionless value
 */
exports.sin = wrapTrigFn(Math.sin);
/**
 * `Math.tan` for `number` measures.
 * @param x an angle
 * @returns a dimensionless value
 */
exports.tan = wrapTrigFn(Math.tan);
/**
 * `Math.atan2` for `number` measures.
 * @param x a length
 * @param y a length
 * @returns an angle
 */
function atan2(y, x) {
    return measure_1.Measure.of(Math.atan2(y.value, x.value), base_1.radians);
}
exports.atan2 = atan2;
/** `Math.sqrt` for `number` measures. Can only be applied to units with even exponents. */
function sqrt(x) {
    return measure_1.Measure.unsafeConstruct(Math.sqrt(x.value), unitValueArithmetic_1.nthRootUnit(x.unit, 2));
}
exports.sqrt = sqrt;
/** `Math.cbrt` for `number` measures. Can only be applied to units with exponents that are multiples of 3. */
function cbrt(x) {
    return measure_1.Measure.unsafeConstruct(Math.cbrt(x.value), unitValueArithmetic_1.nthRootUnit(x.unit, 3));
}
exports.cbrt = cbrt;
/** `Math.pow` for measures of any kind. The power is restricted to integers between -5 and 5 */
function pow(measure, power) {
    return measure.pow(power);
}
exports.pow = pow;
/** Equivalent to `left.plus(right)` */
function add(left, right) {
    return left.plus(right);
}
exports.add = add;
/** Equivalent to `left.minus(right)` */
function subtract(left, right) {
    return left.minus(right);
}
exports.subtract = subtract;
/** Equivalent to `left.times(right)` */
function multiply(left, right) {
    return left.times(right);
}
exports.multiply = multiply;
/** Equivalent to `left.over(right)` */
function divide(left, right) {
    return left.over(right);
}
exports.divide = divide;
/** Sums a list of 1 or more measures, all of the same unit. */
function sum(first) {
    var rest = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        rest[_i - 1] = arguments[_i];
    }
    var result = first;
    for (var _a = 0, rest_1 = rest; _a < rest_1.length; _a++) {
        var measure = rest_1[_a];
        result = result.plus(measure);
    }
    return result;
}
exports.sum = sum;
function wrapUnaryFn(f) {
    return function (x) { return x.unsafeMap(f); };
}
function wrapTrigFn(f) {
    return function (angle) { return measure_1.Measure.dimensionless(f(angle.value)); };
}
function wrapInverseTrigFn(f) {
    return function (angle) { return measure_1.Measure.of(f(angle.value), base_1.radians); };
}
function warpNary(f) {
    return function (first) {
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        return measure_1.Measure.unsafeConstruct(f.apply(void 0, values.apply(void 0, [first].concat(rest))), first.unit);
    };
}
function values() {
    var measures = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        measures[_i] = arguments[_i];
    }
    return measures.map(function (measure) { return measure.value; });
}
//# sourceMappingURL=math.js.map