"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var measure_1 = require("../measure/measure");
var Base = require("../unit/base");
// Dimensionless
/** A measure without any unit */
// HACKHACK: Explicitly type this so we can import GenericMeasure and avoid absolute paths in the generated typings.
exports.Dimensionless = measure_1.Measure.dimensionless(1);
exports.Length = Base.meters;
exports.Mass = Base.kilograms;
exports.Time = Base.seconds;
exports.ElectricCurrent = Base.amperes;
exports.Temperature = Base.kelvin;
exports.AmountOfSubstance = Base.moles;
exports.LuminousIntensity = Base.candelas;
exports.PlaneAngle = Base.radians;
exports.SolidAngle = Base.steradians;
// Derived units
/** 1 / s */
exports.Frequency = exports.Time.inverse();
/** 1 / s^2 */
exports.FrequencyDrift = exports.Time.inverse().squared();
/** 1 / m^2 */
exports.FuelEfficiency = exports.Length.inverse().squared();
/** 1 / m */
exports.Wavenumber = exports.Length.inverse();
/** m^2 */
exports.Area = exports.Length.squared();
/** m^3 */
exports.Volume = exports.Length.cubed();
/** m ⋅ s */
exports.Absement = exports.Length.times(exports.Time);
/** m / s */
exports.Velocity = exports.Length.over(exports.Time);
/** m / s^2 */
exports.Acceleration = exports.Velocity.over(exports.Time);
/** m / s^3 */
exports.Jerk = exports.Acceleration.over(exports.Time);
/** m / s^4 */
exports.Jounce = exports.Jerk.over(exports.Time);
/** m / s^5 */
exports.Crackle = exports.Jounce.over(exports.Time);
/** m^3 / s */
exports.VolumetricFlow = exports.Volume.over(exports.Time);
/** kg / s */
exports.MassFlowRate = exports.Mass.over(exports.Time);
/** kg / m */
exports.LinearDensity = exports.Mass.over(exports.Length);
/** kg / m^2  */
exports.AreaDensity = exports.Mass.over(exports.Area);
/** kg / m^3 */
exports.VolumeDensity = exports.Mass.over(exports.Volume);
/** kg ⋅ m / s^2 */
exports.Force = exports.Mass.times(exports.Acceleration);
/** km ⋅ m / s^3 */
exports.Yank = exports.Force.over(exports.Time);
/** kg / (m ⋅ s^2)  */
exports.Pressure = exports.Force.over(exports.Area);
/** m ⋅ s^2 / kg */
exports.Compressibility = exports.Pressure.inverse();
/** kg / (m ⋅ s) */
exports.DynamicViscosity = exports.Pressure.times(exports.Time);
/** kg / s^2 */
exports.SurfaceTension = exports.Force.over(exports.Length);
/** kg ⋅ m / s  */
exports.Momentum = exports.Force.times(exports.Time);
/** kg ⋅ m^2 */
exports.MomentOfInertia = exports.Mass.times(exports.Area);
/** kg ⋅ m^2 / s^2 */
exports.Energy = exports.Force.times(exports.Length);
/** kg ⋅ m^2 / s^3 */
exports.Power = exports.Energy.over(exports.Time);
/** kg / (m ⋅ s^3) */
exports.PowerDensity = exports.Power.over(exports.Volume);
/** kg ⋅ m^2 / (s^3 ⋅ A) */
exports.Voltage = exports.Power.over(exports.ElectricCurrent);
/** s ⋅ A */
exports.ElectricCharge = exports.ElectricCurrent.times(exports.Time);
/** s ⋅ A / m^3 */
exports.ElectricChargeDensity = exports.ElectricCharge.over(exports.Volume);
/** A / m^2 */
exports.ElectricCurrentDensity = exports.ElectricCurrent.over(exports.Area);
/** s ⋅ A / m^2 */
exports.ElectricDisplacement = exports.ElectricCharge.over(exports.Area);
/** kg ⋅ m / (s^3 ⋅ A) */
exports.EletricFieldStrength = exports.Voltage.over(exports.Length);
/** s^4 ⋅ A^2 / (kg ⋅ m^2) */
exports.ElectricalCapacitance = exports.ElectricCharge.over(exports.Voltage);
/** s^3 ⋅ A / (kg ⋅ m^2) */
exports.ElectricalConductance = exports.ElectricCurrent.over(exports.Voltage);
/** s^3 ⋅ A^2 / (kg ⋅ m^3) */
exports.ElectricalConductivity = exports.ElectricalConductance.over(exports.Length);
/** kg ⋅ m^2 / (s^3 ⋅ A^2) */
exports.ElectricalResistance = exports.Voltage.over(exports.ElectricCurrent);
/** kg ⋅ m^3 / (s^3 ⋅ A^2) */
exports.ElectricalResistivity = exports.ElectricalResistance.times(exports.Length);
/** kg ⋅ m^2 / (s^2 ⋅ A^2) */
exports.ElectricalInductance = exports.ElectricalResistance.times(exports.Time);
/** s ⋅ A / m */
exports.LinearChargeDensity = exports.ElectricCharge.over(exports.Length);
/** s^4 ⋅ A^2 / (kg ⋅ m^3) */
exports.Permittivity = exports.ElectricalCapacitance.over(exports.Length);
/** kg ⋅ m^2 / (s^2 ⋅ A) */
exports.MagneticFlux = exports.Energy.over(exports.ElectricCurrent);
/** kg / (s^2 ⋅ A) */
exports.MagneticFluxDensity = exports.Voltage.times(exports.Time).over(exports.Length.squared());
/** kg ⋅ m / (s^2 ⋅ A^2)  */
exports.MagneticPermeability = exports.ElectricalInductance.over(exports.Length);
/** A / m */
exports.Magnetization = exports.ElectricCurrent.over(exports.Length);
/** s^2 ⋅ A^2 / (kg ⋅ m^2) */
exports.MagneticReluctance = exports.ElectricalInductance.inverse();
/** kg ⋅ m^3 / (s^2 ⋅ A) */
exports.MagneticMoment = exports.MagneticFlux.times(exports.Length);
/** kg ⋅ m / (s^2 ⋅ A) */
exports.MagneticRigidity = exports.MagneticFluxDensity.times(exports.Length);
/** m^2 ⋅ A */
exports.MagneticDipoleMoment = exports.Energy.over(exports.MagneticFluxDensity);
/** s^2 ⋅ A^2 / (kg ⋅ m) */
exports.MagneticSusceptibility = exports.Length.over(exports.ElectricalInductance);
/** kg / s^3  */
exports.Irradiance = exports.Power.over(exports.Area);
/** kg ⋅ m / (s^2 ⋅ K)  */
exports.Entropy = exports.Energy.over(exports.Temperature);
/** m^2 / (s^2 ⋅ K) */
exports.SpecificHeat = exports.Energy.over(exports.Mass.times(exports.Temperature));
/** m^3 / kg  */
exports.SpecificVolume = exports.Volume.over(exports.Mass);
/** kg ⋅ m / (s^3 ⋅ K)  */
exports.ThermalConductivity = exports.Power.over(exports.Length.times(exports.Temperature));
/** s^3 ⋅ K / (kg ⋅ m^2) */
exports.ThermalResistance = exports.Temperature.over(exports.Power);
/** 1 / K */
exports.ThermalExpansionCoefficient = exports.Temperature.inverse();
/** K / m */
exports.ThermalGradient = exports.Temperature.over(exports.Length);
/** kg ⋅ m^2 / (s^2 ⋅ K ⋅ mol) */
exports.MolarEntropy = exports.Entropy.over(exports.AmountOfSubstance);
/** kg ⋅ m^2 / (s^2 ⋅ mol) */
exports.MolarEnergy = exports.Energy.over(exports.AmountOfSubstance);
/** mol / m^3 */
exports.Molarity = exports.AmountOfSubstance.over(exports.Volume);
/** m^3 / mol */
exports.MolarVolume = exports.Volume.over(exports.AmountOfSubstance);
/** mol / kg */
exports.Molality = exports.AmountOfSubstance.over(exports.Mass);
/** kg / mol  */
exports.MolarMass = exports.Mass.over(exports.AmountOfSubstance);
/** s^3 ⋅ A^2 / (kg ⋅ mol) */
exports.MolarConductivity = exports.ElectricalConductance.times(exports.Area).over(exports.AmountOfSubstance);
/** mol / s */
exports.CatalyticActivity = exports.AmountOfSubstance.over(exports.Time);
/** m^3 / (s ⋅ mol) */
exports.CatalyticEfficiency = exports.Volume.over(exports.AmountOfSubstance.times(exports.Time));
/** mol / (m^3 ⋅ s)  */
exports.ReactionRate = exports.CatalyticActivity.over(exports.Volume);
/** m^2 / s^2 */
exports.RadiationDose = exports.Energy.over(exports.Mass);
/** m^2 / s^3 */
exports.RadiationDoseRate = exports.RadiationDose.over(exports.Time);
/** s^2 ⋅ A / kg  */
exports.ElectronMobility = exports.Area.over(exports.Voltage.times(exports.Time));
/** kg ⋅ m^2 / s */
exports.AngularMomentum = exports.Force.times(exports.Length).times(exports.Time);
/** m^2 /s */
exports.SpecificAngularMomentum = exports.AngularMomentum.over(exports.Mass);
/** cd / m^2 */
exports.Luminance = exports.LuminousIntensity.over(exports.Area);
// Angular derived units
/** cd ⋅ sr */
exports.LuminousFlux = exports.LuminousIntensity.times(exports.SolidAngle);
/** cd ⋅ sr / m^2 */
exports.Illuminance = exports.LuminousFlux.over(exports.Area);
/** s ⋅ cd ⋅ sr */
exports.LuminousEnergy = exports.LuminousFlux.times(exports.Time);
/** s ⋅ cd ⋅ sr / m^2 */
exports.LuminousExposure = exports.Illuminance.times(exports.Time);
/** s^3 ⋅ cd ⋅ sr / (kg ⋅ m^2) */
exports.LuminousEfficiency = exports.LuminousFlux.over(exports.Power);
/** kg ⋅ m^2 / (s^3 ⋅ sr) */
exports.RadiantIntensity = exports.Power.over(exports.SolidAngle);
/** kg ⋅ m / (s^3 ⋅ sr)  */
exports.SpectralIntensity = exports.RadiantIntensity.over(exports.Length);
/** kg / (s^3 ⋅ sr) */
exports.Radiance = exports.RadiantIntensity.over(exports.Area);
/** kg / (m ⋅ s^3 ⋅ sr) */
exports.SpectralRadiance = exports.RadiantIntensity.over(exports.Volume);
/** rad / s */
exports.AngularVelocity = exports.PlaneAngle.over(exports.Time);
/** rad / s^2 */
exports.AngularAcceleration = exports.AngularVelocity.over(exports.Time);
//# sourceMappingURL=quantities.js.map